@@ -98,19 +98,25 @@ struct soap_wsa_data
 
 int soap_wsa(struct soap *soap, struct soap_plugin *p, void *arg);
 
+const char *soap_wsa_rand_uuid(struct soap *soap);
+
 int soap_wsa_request(struct soap *soap, const char *id, const char *to, const char *action);
 int soap_wsa_add_From(struct soap *soap, const char *endpoint);
 int soap_wsa_add_NoReply(struct soap *soap);
 int soap_wsa_add_ReplyTo(struct soap *soap, const char *endpoint);
 int soap_wsa_add_FaultTo(struct soap *soap, const char *endpoint);
+int soap_wsa_add_RelatesTo(struct soap *soap, const char *endpoint);
 
 int soap_wsa_check(struct soap *soap);
 int soap_wsa_reply(struct soap *soap, const char *id, const char *action);
+int soap_wsa_fault_subcode(struct soap *soap, int flag, const char *faultsubcode, const char *faultstring, const char *faultdetail);
+int soap_wsa_fault_subcode_action(struct soap *soap, int flag, const char *faultsubcode, const char *faultstring, const char *faultdetail, const char *action);
 int soap_wsa_sender_fault_subcode(struct soap *soap, const char *faultsubcode, const char *faultstring, const char *faultdetail);
+int soap_wsa_sender_fault_subcode_action(struct soap *soap, const char *faultsubcode, const char *faultstring, const char *faultdetail, const char *action);
 int soap_wsa_receiver_fault_subcode(struct soap *soap, const char *faultsubcode, const char *faultstring, const char *faultdetail);
+int soap_wsa_receiver_fault_subcode_action(struct soap *soap, const char *faultsubcode, const char *faultstring, const char *faultdetail, const char *action);
 int soap_wsa_sender_fault(struct soap *soap, const char *faultstring, const char *faultdetail);
 int soap_wsa_receiver_fault(struct soap *soap, const char *faultstring, const char *faultdetail);
-int soap_wsa_fault_subcode(struct soap *soap, int flag, const char *faultsubcode, const char *faultstring, const char *faultdetail);
 
 #if defined(SOAP_WSA_2005)
 int soap_wsa_check_fault(struct soap *soap, SOAP_WSA(FaultCodesType) *fault, const char **info);
